@@ -96,10 +96,18 @@ public class SessionData {
     @NonFinal
     String host;
 
+    @Wither
+    @NonFinal
+    String xHost;
+
     @Wither
     @NonFinal
     String forwardedFor;
 
+    @Wither
+    @NonFinal
+    String forwardedProto;
+
     @Wither
     @NonFinal
     @NonNull
