@@ -104,9 +104,13 @@ public abstract class AbstractConnector<T> implements Connector<T, T> {
         HostConfiguration configuration = sessionData.getConfiguration();
         if (configuration.isForwardHostHeader()) {
             String hostname = sessionData.getHost();
+            String xHostname = sessionData.getXHost();
             if (!Strings.isEmpty(hostname)) {
                 builder.addHeader(RequestUtil.HEADER_HOST, hostname);
             }
+            if (!Strings.isEmpty(xHostname)) {
+                builder.addHeader(RequestUtil.HEADER_X_HOST, xHostname);
+            }
         }
         String forwardedFor = sessionData.getForwardedFor();
         if (!Strings.isEmpty(forwardedFor)) {
@@ -117,6 +121,10 @@ public abstract class AbstractConnector<T> implements Connector<T, T> {
             builder.addHeader(RequestUtil.HEADER_USER_AGENT, userAgent);
         }
         builder.addParameter("session", sessionData.getSession());
+        String trackingId = sessionData.getProperty(SessionData.PROPERTY_TRACKING_ID);
+        if (!Strings.isEmpty(trackingId)) {
+            builder.addParameter("trackingId", trackingId);
+        }
     }
 
     @Override
