@@ -50,6 +50,7 @@
 package com.openexchange.mobile.api.facade.services;
 
 import java.util.Collections;
+import java.util.HashSet;
 import java.util.List;
 
 import com.openexchange.mobile.api.facade.auth.SessionData;
@@ -61,32 +62,53 @@ import com.openexchange.mobile.api.facade.connectors.impl.ConfigConnector;
 import com.openexchange.mobile.api.facade.connectors.impl.CreateFolderConnector;
 import com.openexchange.mobile.api.facade.connectors.impl.DeleteFolderConnector;
 import com.openexchange.mobile.api.facade.connectors.impl.ExamineMultipleFoldersConnector;
+import com.openexchange.mobile.api.facade.connectors.impl.UpdateFolderConnector;
 import com.openexchange.mobile.api.facade.connectors.responses.ConfigResponseMto;
 import com.openexchange.mobile.api.facade.connectors.responses.ExamineResponseMto;
 import com.openexchange.mobile.api.facade.connectors.responses.FoldersResponseMto;
 import com.openexchange.mobile.api.facade.connectors.responses.StringResponseMto;
 import com.openexchange.mobile.api.facade.connectors.responses.mtos.ExamineResultMto;
 import com.openexchange.mobile.api.facade.connectors.responses.mtos.FoldersMto;
+import com.openexchange.mobile.api.facade.exceptions.ApiFacadeException;
 import com.openexchange.mobile.api.facade.models.Account;
 import com.openexchange.mobile.api.facade.models.Config;
 import com.openexchange.mobile.api.facade.models.CreatedFolder;
 import com.openexchange.mobile.api.facade.models.ExamineResult;
 import com.openexchange.mobile.api.facade.models.Folder;
 import com.openexchange.mobile.api.facade.models.MailAccountData;
+import com.openexchange.mobile.api.facade.models.UpdatedFolder;
+import com.openexchange.mobile.api.facade.models.Folder.Type;
 import com.openexchange.mobile.api.facade.utils.ListUtil;
 import com.openexchange.mobile.api.facade.utils.MapFunction;
 
 import lombok.RequiredArgsConstructor;
 import lombok.Setter;
+import lombok.extern.slf4j.Slf4j;
 
 @RequiredArgsConstructor
+@Slf4j
 public class FoldersService {
 
+    private static final HashSet<String> ERROR_CODES_AUTHENTICATION = new HashSet<String>();
+
     private final ConnectorFactoryFactory connectorFactoryFactory;
 
     @Setter
     private AccountsService accountsService;
 
+    static {
+        ERROR_CODES_AUTHENTICATION.add("MSG-0091");
+        ERROR_CODES_AUTHENTICATION.add("MSG-0114");
+        ERROR_CODES_AUTHENTICATION.add("MSG-1000");
+        ERROR_CODES_AUTHENTICATION.add("MSG-1001");
+        ERROR_CODES_AUTHENTICATION.add("MSG-1031");
+        ERROR_CODES_AUTHENTICATION.add("OAUTH-0013");
+        ERROR_CODES_AUTHENTICATION.add("OAUTH-0042");
+        ERROR_CODES_AUTHENTICATION.add("OAUTH-0043");
+        ERROR_CODES_AUTHENTICATION.add("OAUTH-0044");
+        ERROR_CODES_AUTHENTICATION.add("pop3-2071");
+    }
+
     public CreatedFolder createFolder(SessionData sessionData, String parentFolderId, String name) {
         ConnectorFactory connectorFactory = connectorFactoryFactory.getConnectorFactory();
         CreateFolderConnector connector = connectorFactory.getCreateFolderConnector(sessionData, parentFolderId, name);
@@ -95,6 +117,14 @@ public class FoldersService {
         return new CreatedFolder(folderId, name);
     }
 
+    public UpdatedFolder updateFolder(SessionData sessionData, String folderId, String newParentFolderId, String newName) {
+        ConnectorFactory connectorFactory = connectorFactoryFactory.getConnectorFactory();
+        UpdateFolderConnector connector = connectorFactory.getUpdateFolderConnector(sessionData, folderId, newParentFolderId, newName);
+        StringResponseMto response = connector.execute();
+        String newFolderId = UpdateFolderConnector.map(response);
+        return new UpdatedFolder(newFolderId);
+    }
+
     public void deleteFolder(SessionData sessionData, String folderId) {
         ConnectorFactory connectorFactory = connectorFactoryFactory.getConnectorFactory();
         DeleteFolderConnector connector = connectorFactory.getDeleteFolderConnector(sessionData, folderId);
@@ -148,6 +178,8 @@ public class FoldersService {
         List<Folder> folders = retrieveFolders(connectorFactory, sessionData, account.getRootFolderId(), account.getArchiveFolderId(), language);
         if (!account.isPop3Account()) {
             examineFolders(connectorFactory, sessionData, folders);
+        } else {
+            examineResetFolders(folders);
         }
         account = account.withFolders(folders);
         adjustFolderParents(account, account.isPrimaryAccount() && isNamespaceSet);
@@ -163,8 +195,21 @@ public class FoldersService {
     private List<Folder> retrieveFolders(ConnectorFactory connectorFactory, SessionData sessionData, String rootFolderId, String archiveFolderId, String language) {
         boolean includeNonPrivateFolders = sessionData.getConfiguration().isIncludeNonPrivateFolders();
         AllFoldersConnector connector = connectorFactory.getAllFoldersConnector(sessionData, rootFolderId, language);
-        FoldersResponseMto response = connector.execute();
-        FoldersMto data = response.getData();
+        FoldersMto data;
+        try {
+            FoldersResponseMto response = connector.execute();
+            data = response.getData();
+        } catch (ApiFacadeException e) {
+            String errorCode = e.getErrorCode();
+            log.info("Retrieve folders failed with code: " + errorCode);
+            // OMFACADE-633: Throw authentication errors directly back to the clients.
+            if (ERROR_CODES_AUTHENTICATION.contains(errorCode)) {
+                throw e;
+            }
+            // Return fake INBOX folder to not crash older iOS clients.
+            Folder inboxFolder = new Folder(rootFolderId + "/INBOX", rootFolderId, "INBOX", Type.INBOX);
+            return Collections.singletonList(inboxFolder);
+        }
         return data != null
                 ? AllFoldersConnector.map(data.getPrivateFolders(), archiveFolderId, includeNonPrivateFolders)
                 : Collections.<Folder>emptyList();
@@ -201,6 +246,16 @@ public class FoldersService {
         return new ExamineResult(folderId, changed, newValidity, newNext);
     }
 
+    private void examineResetFolders(List<Folder> folders) {
+        for (int index = 0; index < folders.size(); index++) {
+            Folder folder = folders.get(index);
+            folder.setValidity(null);
+            folder.setNext(null);
+            folder.setTotal(-1);
+            folder.setUnread(-1);
+        }
+    }
+
     private void adjustFolderParents(Account account, boolean isNamespaceSet) {
         if (!isNamespaceSet) {
             return;
@@ -232,4 +287,5 @@ public class FoldersService {
             }
         }
     }
+
 }
